function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

/**
 * useAsync
 * 我们先预设一下场景：
 * 1. 一个 useAsync 会被触发很多次，发多次请求。
 *    1.1 有些时候我们只希望保存最新请求的状态，也就是新的请求一定会覆盖旧的请求。
 *    1.2 有些时候我们希望保存每一次请求的状态，比如我们一般执行“删除”操作，同时删除 n 条，我们并不希望将请求合并。
 * 2. 如果有轮询，并且同时触发了多次 run，一定是以最后一次请求为基准的。也就是，新的请求会取消之前的轮询。
 *
 * 基于以上的假设，我们看下实现原理
 * history & fetchKey: 以 key 分类，记录所有的请求。
 *    - history 是一个 object，会记录每一类的请求状态（注意不是每一次，是每一类）
 *    - 默认情况下 history 只存在一类：DEFAULT_KEY
 *    - 用户可以通过 fetchkey 来设置当前请求的分类。fetchKey 是一个函数，接收当前请求的 params。
 *    - ！！！ 默认 data/error/loading 等，等于最新的 history，通过 newstKey 来记录哪个是最新的请求。
 * count: 每一次请求，都有一个编号。
 *    - 只有最新的请求，在结束后，才允许触发轮询。
 *    - 每一类（同一 key）请求，只有最新的才会生效。也就是同一类请求，也会存在覆盖关系。通过 runKeyCount 记录 key 与 count 的对应关系。
 */
import { useCallback, useEffect, useRef, useState } from 'react';
import { isDocumentVisible } from './utils';
import { getCache, setCache } from './utils/cache';
import useDebounce from './utils/useDebounce';
import useLimitFn from './utils/useLimitFn';
import useThrottle from './utils/useThrottle';
import useUpdateEffect from './utils/useUpdateEffect';
import subscribeFocus from './utils/windowFocus';
import subscribeVisible from './utils/windowVisible';
var DEFAULT_KEY = 'UMIJS_USE_API_DEFAULT_KEY';

function useAsync(service, options) {
  var _options = options || {};

  var _options$refreshDeps = _options.refreshDeps,
      refreshDeps = _options$refreshDeps === void 0 ? [] : _options$refreshDeps,
      _options$extraCacheDe = _options.extraCacheDeps,
      extraCacheDeps = _options$extraCacheDe === void 0 ? [] : _options$extraCacheDe,
      _options$manual = _options.manual,
      manual = _options$manual === void 0 ? false : _options$manual,
      onSuccess = _options.onSuccess,
      onError = _options.onError,
      _options$pollingInter = _options.pollingInterval,
      pollingInterval = _options$pollingInter === void 0 ? 0 : _options$pollingInter,
      _options$pollingWhenH = _options.pollingWhenHidden,
      pollingWhenHidden = _options$pollingWhenH === void 0 ? true : _options$pollingWhenH,
      _options$defaultParam = _options.defaultParams,
      defaultParams = _options$defaultParam === void 0 ? [] : _options$defaultParam,
      _options$refreshOnWin = _options.refreshOnWindowFocus,
      refreshOnWindowFocus = _options$refreshOnWin === void 0 ? false : _options$refreshOnWin,
      _options$focusTimespa = _options.focusTimespan,
      focusTimespan = _options$focusTimespa === void 0 ? 5000 : _options$focusTimespa,
      fetchKey = _options.fetchKey,
      cacheKey = _options.cacheKey,
      _options$debounceInte = _options.debounceInterval,
      debounceInterval = _options$debounceInte === void 0 ? 0 : _options$debounceInte,
      _options$throttleInte = _options.throttleInterval,
      throttleInterval = _options$throttleInte === void 0 ? 0 : _options$throttleInte; // 是否卸载

  var unmountFlag = useRef(false); // 屏幕 visible 后，是否需要开启定时器

  var pollingWhenVisibleFlag = useRef(false); // 轮询定时器

  var timerRef = useRef(); // 防止调用函数时，产生 capture value 问题

  var serviceRef = useRef(service);
  serviceRef.current = service;
  var onSuccessRef = useRef(onSuccess);
  onSuccessRef.current = onSuccess;
  var onErrorRef = useRef(onError);
  onErrorRef.current = onError;
  var fetchKeyRef = useRef(fetchKey);
  fetchKeyRef.current = fetchKey;
  var formatResultRef = useRef();

  if ("formatResult" in _options) {
    formatResultRef.current = _options.formatResult;
  } // 时序控制，每一次请求都有一个唯一的编号


  var count = useRef(0); // 每一类请求都有一个唯一的 Key, newstKey 代表最新的那一类的 key
  // key 是由用户输入的 fetchKey 生成的

  var newstKey = useRef(DEFAULT_KEY); // key 与 count 的对应关系，每一类 key 只有最新的 count 才有效
  // {[key:string]: number}

  var runKeyCount = useRef({});
  var getCacheGroupKey = useCallback(function () {
    if (!cacheKey) {
      return null;
    }

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return JSON.stringify([cacheKey].concat(args, _toConsumableArray(refreshDeps), _toConsumableArray(extraCacheDeps)));
  }, [cacheKey, refreshDeps, extraCacheDeps]); // 请求历史

  var _useState = useState(function () {
    var initCacheData; // 如果是自动执行的，初始化的时候读缓存的数据

    if (!manual && cacheKey) {
      var cacheRes = getCache(getCacheGroupKey.apply(void 0, _toConsumableArray(defaultParams)));

      if (cacheRes) {
        initCacheData = formatResultRef.current ? formatResultRef.current(cacheRes) : cacheRes;
      }
    }

    return _defineProperty({}, DEFAULT_KEY, {
      params: [],
      data: initCacheData,
      error: undefined,
      loading: !manual,
      cancel: function cancel() {
        cancelProxyRef.current(DEFAULT_KEY);
      },
      refresh: function refresh() {
        refreshRef.current(DEFAULT_KEY);
      }
    });
  }),
      _useState2 = _slicedToArray(_useState, 2),
      history = _useState2[0],
      setHistory = _useState2[1];

  var historyRef = useRef(history);
  historyRef.current = history;
  var runProxyRef = useRef();
  var cancelProxyRef = useRef();
  var refreshRef = useRef(); // refresh 某个 history

  var refresh = useCallback(function (key) {
    runProxyRef.current.apply(runProxyRef, _toConsumableArray(historyRef.current[key].params));
  }, []);
  refreshRef.current = refresh;
  /* 根据 key，取消某一类 history */

  var cancel = useCallback(function (key) {
    // count 为 -1，就不会处理响应结果了，相当于取消。因为正在执行的请求, count 永远不可能等于 -1。
    runKeyCount.current[key] = -1;
    setHistory(function (s) {
      s[key] = _objectSpread({}, s[key], {
        loading: false
      });
      return _objectSpread({}, s);
    });
  }, []);
  var run = useCallback(function () {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    // 取消已有定时器
    if (timerRef.current) {
      clearTimeout(timerRef.current);
    } // 请求编号加一


    count.current += 1; // 闭包存储当前请求 count

    var runCount = count.current; // 读取当前最新 newstKey

    if (fetchKeyRef.current) {
      newstKey.current = fetchKeyRef.current.apply(fetchKeyRef, args);
    } // 闭包存储当前请求 分类 key


    var runKey = newstKey.current; // 记录该 runKey 下最新的 count，同一个 runKey，老的 count 均需要废弃

    runKeyCount.current[runKey] = runCount; // 从缓存读数据

    var cacheData;
    var cacheGroupKey = getCacheGroupKey.apply(void 0, args);

    if (cacheGroupKey) {
      var cacheRes = getCache(cacheGroupKey);

      if (cacheRes) {
        cacheData = formatResultRef.current ? formatResultRef.current(cacheRes) : cacheRes;
      }
    }

    setHistory(function (s) {
      var _s$runKey, _s$runKey2;

      s[runKey] = {
        data: cacheData || ((_s$runKey = s[runKey]) === null || _s$runKey === void 0 ? void 0 : _s$runKey.data) || undefined,
        error: ((_s$runKey2 = s[runKey]) === null || _s$runKey2 === void 0 ? void 0 : _s$runKey2.error) || undefined,
        loading: true,
        params: args,
        cancel: function cancel() {
          cancelProxyRef.current(runKey);
        },
        refresh: function refresh() {
          refreshRef.current(runKey);
        }
      };
      return _objectSpread({}, s);
    });
    return serviceRef.current.apply(serviceRef, args).then(function (res) {
      // 同一个 runKey，只有最新的 count 才会响应 
      if (!unmountFlag.current && runCount === runKeyCount.current[runKey]) {
        if (cacheGroupKey) {
          setCache(cacheGroupKey, res);
        }

        var formattedResult = formatResultRef.current ? formatResultRef.current(res) : res;
        setHistory(function (s) {
          s[runKey] = _objectSpread({}, s[runKey] || {}, {
            loading: false,
            error: undefined,
            data: formattedResult
          });
          return _objectSpread({}, s);
        });

        if (onSuccessRef.current) {
          onSuccessRef.current(formattedResult, args);
        }

        return formattedResult;
      }
    }).catch(function (error) {
      if (!unmountFlag.current && runCount === runKeyCount.current[runKey]) {
        setHistory(function (s) {
          s[runKey] = _objectSpread({}, s[runKey] || {}, {
            loading: false,
            error: error,
            data: undefined
          });
          return _objectSpread({}, s);
        });

        if (onErrorRef.current) {
          onErrorRef.current(error, args);
        }

        throw error;
      }
    }).finally(function () {
      // 只有最新的请求，不管是哪个分类，才有资格 setTimeout
      if (!unmountFlag.current && pollingInterval && runCount === count.current) {
        // 如果屏幕隐藏，并且 !pollingWhenHidden, 则停止轮询，并记录 flag，等 visible 时，继续轮询
        if (!isDocumentVisible() && !pollingWhenHidden) {
          pollingWhenVisibleFlag.current = true;
          return;
        }

        timerRef.current = setTimeout(function () {
          refreshRef.current(runKey);
        }, pollingInterval);
      }
    });
  }, [pollingInterval, pollingWhenHidden, getCacheGroupKey]);
  var runDebounce = useDebounce(run, debounceInterval);
  var runThrottle = useThrottle(run, throttleInterval);
  var runProxy = useCallback(function () {
    if (debounceInterval) {
      runDebounce.run.apply(runDebounce, arguments);
      return;
    }

    if (throttleInterval) {
      runThrottle.run.apply(runThrottle, arguments);
      return;
    }

    run.apply(void 0, arguments);
  }, [run, runDebounce, runThrottle]);
  runProxyRef.current = runProxy;
  var cancelProxy = useCallback(function (key) {
    if (runDebounce) {
      runDebounce.cancel();
    }

    if (runThrottle) {
      runThrottle.cancel();
    }

    cancel(key);
  }, [cancel, runDebounce, runThrottle]);
  cancelProxyRef.current = cancelProxy;
  /*------------- polling start ---------------*/
  // 停止轮询

  var stopPolling = useCallback(function () {
    // 取消当前正在等待的定时器
    if (timerRef.current) {
      clearTimeout(timerRef.current);
    } // 取消即将来临的定时器。因为只有最新的 count，在请求结束后，才允许触发定时器。count+1 后，就没任何请求是最新的了。


    count.current += 1;
  }, []);
  var rePolling = useCallback(function () {
    if (pollingWhenVisibleFlag.current) {
      pollingWhenVisibleFlag.current = false;
      timerRef.current = setTimeout(function () {
        refreshRef.current(newstKey.current);
      }, pollingInterval);
    }
  }, []);
  useEffect(function () {
    if (pollingInterval) {
      return subscribeVisible(rePolling);
    }
  }, []);
  /*------------- polling end ---------------*/

  /*------------- refreshOnFocus start ---------------*/

  var reFreshOnFocus = useLimitFn(function () {
    refreshRef.current(newstKey.current);
  }, focusTimespan);
  useEffect(function () {
    if (refreshOnWindowFocus) {
      return subscribeFocus(reFreshOnFocus);
    }
  }, [reFreshOnFocus]);
  /*------------- refreshOnFocus end ---------------*/

  /*------------- mutate start ---------------*/
  // 突变

  var mutate = useCallback(function (newData) {
    setHistory(function (s) {
      s[newstKey.current].data = newData;
      return _objectSpread({}, s);
    });
  }, []);
  /*------------- mutate end ---------------*/

  /*------------- mount&unmount start ---------------*/
  // 第一次默认执行

  useEffect(function () {
    if (!manual) {
      // 第一次默认执行，可以通过 defaultParams 设置参数
      runProxy.apply(void 0, _toConsumableArray(defaultParams));
    }
  }, []); //  refreshDeps 变化，使用最新的参数重新执行

  useUpdateEffect(function () {
    if (!manual) {
      runProxyRef.current.apply(runProxyRef, _toConsumableArray(historyRef.current[newstKey.current].params));
    }
  }, _toConsumableArray(refreshDeps)); // 卸载组件触发

  useEffect(function () {
    return function () {
      if (timerRef.current) {
        clearTimeout(timerRef.current);
      }

      unmountFlag.current = true;
    };
  }, []);
  /*------------- mount&unmount end ---------------*/

  return _objectSpread({}, history[newstKey.current], {
    stopPolling: stopPolling,
    run: runProxy,
    mutate: mutate,
    history: history
  });
}

export default useAsync;