import { PaginationConfig } from 'antd/lib/pagination';
import { SorterResult } from 'antd/lib/table';
import { DependencyList } from 'react';
declare type noop = (...args: any[]) => void;
export interface BaseResult<T, K extends any[]> {
    loading: boolean;
    data: T | undefined;
    error: Error | string | undefined;
    run: (...args: K) => Promise<T>;
    params: K | undefined;
    cancel: noop;
    refresh: noop;
    stopPolling: noop;
    mutate: (data: T) => void;
    history: {
        [key: string]: {
            loading: boolean;
            data: T | undefined;
            error: Error | string | undefined;
            params: K | undefined;
            cancel: noop;
            refresh: noop;
        };
    };
}
export declare type BaseOptions<R, P extends any[]> = {
    refreshDeps?: DependencyList;
    manual?: boolean;
    onSuccess?: (data: R, params: P) => void;
    onError?: (e: Error, params: P) => void;
    defaultParams?: P;
    pollingInterval?: number;
    pollingWhenHidden?: boolean;
    fetchKey?: (...args: P) => string | number;
    paginated?: false;
    refreshOnWindowFocus?: boolean;
    focusTimespan?: number;
    cacheKey?: string;
    extraCacheDeps?: DependencyList;
    debounceInterval?: number;
    throttleInterval?: number;
};
export declare type OptionsWithFormat<R, P extends any[], U, UU extends U> = {
    formatResult: (res: R) => U;
} & BaseOptions<UU, P>;
export declare type Options<R, P extends any[], U, UU extends U> = BaseOptions<R, P> | OptionsWithFormat<R, P, U, UU>;
export interface PaginatedParams<Item> {
    current: number;
    pageSize: number;
    sorter?: SorterResult<Item>;
    filters?: Record<keyof Item, string[]>;
}
export interface PaginatedFormatReturn<Item> {
    total: number;
    list: Item[];
    [key: string]: any;
}
export interface PaginatedResult<Item> extends BaseResult<PaginatedFormatReturn<Item>, [PaginatedParams<Item>]> {
    pagination: {
        current: number;
        pageSize: number;
        total: number;
        totalPage: number;
        onChange: (current: number, pageSize: number) => void;
        changeCurrent: (current: number) => void;
        changePageSize: (pageSize: number) => void;
    };
    tableProps: {
        dataSource: Item[];
        loading: boolean;
        onChange: (pagination: PaginationConfig, filters?: Record<keyof Item, string[]>, sorter?: SorterResult<Item>) => void;
        pagination: {
            current: number;
            pageSize: number;
            total: number;
        };
    };
    sorter?: SorterResult<Item>;
    filters?: Record<keyof Item, string[]>;
}
export interface BasePaginatedOptions<U> extends Omit<BaseOptions<PaginatedFormatReturn<U>, [PaginatedParams<U>]>, 'paginated'> {
    paginated: true;
    defaultPageSize?: number;
}
export interface PaginatedOptionsWithFormat<R, Item, U> extends Omit<BaseOptions<PaginatedFormatReturn<U>, [PaginatedParams<U>]>, 'paginated'> {
    paginated: true;
    defaultPageSize?: number;
    formatResult: (data: R) => PaginatedFormatReturn<Item>;
}
export {};
