/**
 * useAsync
 * 我们先预设一下场景：
 * 1. 一个 useAsync 会被触发很多次，发多次请求。
 *    1.1 有些时候我们只希望保存最新请求的状态，也就是新的请求一定会覆盖旧的请求。
 *    1.2 有些时候我们希望保存每一次请求的状态，比如我们一般执行“删除”操作，同时删除 n 条，我们并不希望将请求合并。
 * 2. 如果有轮询，并且同时触发了多次 run，一定是以最后一次请求为基准的。也就是，新的请求会取消之前的轮询。
 *
 * 基于以上的假设，我们看下实现原理
 * history & fetchKey: 以 key 分类，记录所有的请求。
 *    - history 是一个 object，会记录每一类的请求状态（注意不是每一次，是每一类）
 *    - 默认情况下 history 只存在一类：DEFAULT_KEY
 *    - 用户可以通过 fetchkey 来设置当前请求的分类。fetchKey 是一个函数，接收当前请求的 params。
 *    - ！！！ 默认 data/error/loading 等，等于最新的 history，通过 newstKey 来记录哪个是最新的请求。
 * count: 每一次请求，都有一个编号。
 *    - 只有最新的请求，在结束后，才允许触发轮询。
 *    - 每一类（同一 key）请求，只有最新的才会生效。也就是同一类请求，也会存在覆盖关系。通过 runKeyCount 记录 key 与 count 的对应关系。
 */
import { BaseOptions, BaseResult, OptionsWithFormat } from './types';
declare function useAsync<R, P extends any[], U, UU extends U = any>(service: (...args: P) => Promise<R>, options: OptionsWithFormat<R, P, U, UU>): BaseResult<U, P>;
declare function useAsync<R, P extends any[]>(service: (...args: P) => Promise<R>, options?: BaseOptions<R, P>): BaseResult<R, P>;
export default useAsync;
